# File Maker for blue image mask 8bit & 16bit Version
# Ver. 0.20
# (c)2020-2021 Ohnishi, Yasuo
from ij import IJ 
from ij import plugin
from ij.io import DirectoryChooser
from ij.io import OpenDialog
from ij.gui import GenericDialog
import os
#----------------------------------------------
### function of Value Converting depend on Mode
def convValue(Value, Mode):
#   Converting Luminance Value each in 8, 16, 32bit mode
#    print("value org: ", Value)
    if Mode == "16-bit":
        Value = Value * 256
    elif Mode == "32-bit":
        Value = Value * 256 * 256
    else:
        Value = Value
#   print("Converted value: ",Value, Mode)
    return Value
#----------------------------------------------
### function of Windows Name Converting depend on Mode
def convWinName(Name, filename, Mode):
#   Converting Windows Name each in 8, 16, 32bit mode
    if Mode == "8-bit":
        if Name == "C1-":
            Name = filename + " (red)"
        elif Name == "C2-":
            Name = filename + " (green)"
        elif Name == "C3-":
            Name = filename + " (blue)"
    else:
        Name = Name + filename
#   print ("Windows Names:",Name)
    return Name
#----------------------------------------------
### RGB Manipulation 
def RGBmanipulation(folder,filename,parm):
#  input Parameters
    if parm is not None:
 # if parm is not null object, then...
#        Mode, parm_Binary, parm_Distance, parm_Dark = parm
#        print(Mode, parm_Binary, parm_Distance, parm_Dark) 
        Mode, Image = parm
#     Mode: Image Format 16-bit or 8-bit
#     parm_Binary: Threshold of fore / background
#     parm_Distance: Lightness of Distance adjusting Layer
    if Mode == "8-bit":
        IJ.open(folder + filename)
    else:
        IJ.run("Bio-Formats Importer", "open=\'"+ folder + filename + "\' autoscale color_mode=Default rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT")
    IJ.run("Split Channels")
    IJ.run(Mode)
# R ch.
    IJ.selectWindow(convWinName("C1-", filename, Mode))
    imp0 =  IJ.getImage()
    IJ.saveAsTiff(imp0, folder + filename +"_Rtmp.tif")
 # Window named filename +"_Rtmp.tif"
# G ch.
    IJ.selectWindow(convWinName("C2-", filename, Mode))
    imp1 =  IJ.getImage()
    IJ.saveAsTiff(imp1, folder + filename +"_Gtmp.tif")
 # Window named filename +"_Gtmp.tif"
# B ch. 
    IJ.selectWindow(convWinName("C3-", filename, Mode))
    imp2 =  IJ.getImage()
    IJ.saveAsTiff(imp2, folder + filename +"_Btmp.tif")
 # Window named filename +"_Btmp.tif"
# G ch. x 0.5
    IJ.selectWindow(filename +"_Gtmp.tif")
    IJ.run("Copy")
    IJ.run("Internal Clipboard")
 # Open "Clipboard" Window as G x 0.5 Image
    IJ.run("Multiply...", "value=0.5")
    impG05 = IJ.getImage()
# R ch. x 0.5
    IJ.selectWindow(filename +"_Rtmp.tif")
    IJ.run("Copy")
    IJ.run("Internal Clipboard")
# Open "Clipboard-1" Window as R x 0.5 Image
    IJ.run("Multiply...", "value=0.5")
    impR05 = IJ.getImage()
# R x 0.5 + G x 0.5
    ic = plugin.ImageCalculator()
    impRG = ic.run("Add create", impR05, impG05)
    IJ.saveAsTiff(impRG, folder + filename +"_G+Rtmp.tif") # No Window made
# Close Windows
    IJ.selectWindow("Clipboard")
    IJ.run("Close") # Close G x 0.5 Window
    IJ.selectWindow("Clipboard-1")
    IJ.run("Close") # Close R x 0.5 Window
    IJ.selectWindow(filename +"_Rtmp.tif")
    IJ.run("Close")
    IJ.selectWindow(filename +"_Gtmp.tif")
    IJ.run("Close")
#   Window filename +"_Btmp.tif" continue to be open
    IJ.open(folder + filename +"_G+Rtmp.tif")
#   Window filename +"_G+Rtmp.tif" continue to be open
# (R x 0.5 + G x 0.5) - B
    ic = plugin.ImageCalculator()
    for num in range(-1, 3):
# Meke Blue Image Files in 4 Threshold B-((R+G)/2), B-((R+G)/2+10), B-((R+G)/2-10), B-((R+G)/2-20) 
        IJ.selectWindow(filename + "_G+Rtmp.tif") # Select the R + G image in window
        IJ.run("Copy")
        IJ.run("Internal Clipboard")  # Open "Clipboard" Window < R + G Image
        V = str(abs(num) * 10)
 #set Threshold Value (Nominal)
        V256 = str(convValue(abs(num) * 10, Mode))
 #set Threshold Value (depend on "Mode")
        if num > 0:  # Set the Threshold in R + G Image
            print num, "Subtract", V
            IJ.run("Subtract...", "value="+ V256)
            V= "+" + V
        else:
            print num, "Add", V
            IJ.run("Add...", "value="+ V256)
            V= "-" + V
        imp5 = IJ.getImage()
        imp6 = ic.run("Subtract create", imp2, imp5) #imp2:B imp5:G+R
        imp6.show()  # Show the subtracted image in window
        IJ.selectWindow("Result of " + filename +"_Btmp.tif") # Selects the B - (R + G) image in window
        IJ.run("Multiply...", "value=4")
 # B - (R + G) image x 4
        imp8 = IJ.getImage() #imp8 < Normal Image of B-(R+G)/2
        IJ.saveAsTiff(imp8, folder + filename +"_BMask_Thr"+ V + "_x4.tif") # Save the Subtracted image x 4
        IJ.run("Invert")
        imp9 = IJ.getImage() #imp9 < Inverted Image of B-(R+G)/2
        IJ.saveAsTiff(imp9, folder + filename +"_BMask_Thr"+ V + "_x4-Inv.tif") # Save the Subtracted image x 4 x Invert
        IJ.selectWindow(filename +"_BMask_Thr"+ V + "_x4-Inv.tif") # Select the Inverse Image window
        IJ.run("Close") # Close the Inverse Image Window
        if Image == "inverse": #Save Inverse Image only
            os.remove(folder + filename +"_BMask_Thr"+ V + "_x4.tif")
        if Image == "normal": #Save Normal Image only
            os.remove(folder + filename +"_BMask_Thr"+ V + "_x4-Inv.tif")
        IJ.selectWindow("Clipboard") # Close the Internal Clipboard Image Window
        IJ.run("Close") # Close Window
    IJ.selectWindow(filename +"_G+Rtmp.tif") # Select the G+R Image window
    IJ.run("Close") # Close the G+R Image Window 
    IJ.selectWindow(filename +"_Btmp.tif") # Select the B Image window
    IJ.run("Close") # Close the G+R Image Window 
    os.remove(folder + filename +"_Rtmp.tif")
    os.remove(folder + filename +"_Gtmp.tif")
    os.remove(folder + filename +"_Btmp.tif")
    os.remove(folder + filename +"_G+Rtmp.tif")
#   End of def RGBmanipulation 
#----------------------------------------------
### Get parameters from the input DialogBox 
def getParameters():
	gd = GenericDialog("Enter Parameters")
	mode = ["16-bit","8-bit"]
	gd.addChoice("Image Format (16bit or 8bit [x 3ch.])", mode, mode[0])
	image = ["both","inverse","normal"]
	gd.addChoice("Output Image Type", image, image[0])
	gd.showDialog()

	if gd.wasCanceled():  
		parm_Mode   = "Stop"
		IJ.log("Canceled")
		return str(parm_Mode),image[0]
		
	parm_Mode   = gd.getNextChoice()
	parm_Image   = gd.getNextChoice()
	return str(parm_Mode), str(parm_Image)
#----------------------------------------------
##### Main Program
od = OpenDialog("Choose a file")
folder = od.getDirectory()
file = od.getFileName()
path = folder + file
IJ.log(path)
IJ.log(folder)
IJ.log(file)
parm = getParameters()
print ("Parm[0]=", parm[0])
if parm[0] <> "Stop":
    RGBmanipulation(folder,file,parm)
    IJ.log("Finish")
    IJ.run("Quit")
